package drew.figure;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.*;
import math.*;
import drew.util.*;
import java.beans.*;


public class FigureAxes extends JPanel 
  implements MouseListener, MouseMotionListener, java.io.Serializable
{
  static final long serialVersionUID = -264799284283563063L;

  private Vector figures = new Vector();

  //public void setFigures(int i, Figure f) { figures.setElementAt(f, i); }
  //public Figure getFigures(int i) { return (Figure) figures.elementAt(i); }
  public void setFigures(Vector v) { figures = v; }
  public Vector getFigures() { return figures;}

  public FigureAxes()
  {
    setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
    setPreferredSize(new Dimension(210,200));
    addMouseListener(this);
    addMouseMotionListener(this);
  }


  Rectangle graphRect = new Rectangle();

  int titleWidth, titleHeight;

  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);


    Graphics g2 = g.create();
    if (title != null)
      {
	if (titleFont == null)
	  titleFont = g2.getFont();
	else
	  g2.setFont(titleFont);
	FontMetrics metric = getFontMetrics(g2.getFont());

	titleHeight = metric.getHeight();
	titleWidth = metric.stringWidth(title);
	titleDescent = metric.getMaxDescent();
	titleHeight += titleGap;
      }

    

    int titleOffset = 0;
    if (title != null)
      titleOffset = titleHeight;

    if (usePixelUnits)
      {
	double w = Math.abs(maxX-minX);
	double h = Math.abs(maxY-minY);
	if (showAxes)
	  setSize((int) (w + axesNumberWidth), (int) (h + titleOffset + axesNumberHeight));
	else
	  setSize((int) w, (int) h + titleOffset);
      }

    if (showAxes)
      {
	graphRect.x = axesNumberWidth;
	graphRect.y = titleOffset;
	graphRect.width = getSize().width - graphRect.x;
	graphRect.height = getSize().height - axesNumberHeight - titleOffset;
	drawAxes(g, g2);
      }
    else
      {
	graphRect.x = 0;
	graphRect.y = titleOffset;
	graphRect.width = getSize().width;
	graphRect.height = getSize().height - titleOffset;
      }

    if (graphRect.height != oldGraphRectHeight)
      changeAxes();
    if (graphRect.width != oldGraphRectWidth)
      changeAxes();
    oldGraphRectHeight = graphRect.height;
    oldGraphRectWidth = graphRect.width;

    g.translate(graphRect.x,graphRect.y);
    g.setColor(graphBackgroundColor);
    g.fillRect(1, 0, graphRect.width, graphRect.height);

    g = g.create(1,0,graphRect.width-1,graphRect.height);

    for (int i = 0; i < figures.size(); i++)
      {
	Figure f = (Figure) figures.elementAt(i);
	
	if (!f.toldAxes)
	  f.setAxes(minX, maxX, minY, maxY, graphRect);
	f.toldAxes = true;
	f.register(this);
	f.paint(g);
      }


    if (drawingBox) // for the case where we're getting paint calls while drawing a box
      {  // doesn't work too well without double buffering
	oldRubberBox = null;
	Rectangle r = fixBox(dragStartX, dragStartY, dragFinishX, dragFinishY);
	drawRubberBox(r);
      }


  }

  int oldGraphRectWidth, oldGraphRectHeight;

  public void changeAxes()
  {
    for (int i = 0; i < figures.size(); i++)
      {
	Figure f = (Figure) figures.elementAt(i);
	f.setAxes(minX, maxX, minY, maxY, graphRect);
      }
  }
  

  private void drawAxes(Graphics g, Graphics g2)
  {
    // along the horizontal (x) axis
    g.drawLine(graphRect.x, graphRect.y + graphRect.height, 
	       graphRect.x + graphRect.width, 
	       graphRect.y + graphRect.height);

    g.drawString(drew.util.Format.formatDouble(minX,5),
		 graphRect.x, 
		 graphRect.y + graphRect.height + axesNumberHeight);
    g.drawString(drew.util.Format.formatDouble(maxX,5),
		 graphRect.x + graphRect.width - axesNumberWidth,
		 graphRect.y + graphRect.height + axesNumberHeight);

    // along the vertical (y) axis
    g.drawLine(graphRect.x, graphRect.y, graphRect.x, 
	       graphRect.y + graphRect.height);
    g.drawString(drew.util.Format.formatDouble(maxY,5),
		 0,
		 axesNumberHeight + graphRect.y);
    g.drawString(drew.util.Format.formatDouble(minY,5), //Double.toString(minY),
		 0,
		 graphRect.height + graphRect.y);

    if (title != null)
      {

	int titleX;
	if (centerTitle)
	  titleX = (graphRect.width + graphRect.x)/2 - titleWidth/2;
	else
	  titleX = axesNumberWidth;
	
	g2.drawString(title, titleX, titleHeight - titleDescent - titleGap);
      }


  }
  

  int titleGap = 2;
  int titleDescent;


  private double minX = 0, minY = 0, maxX = 1, maxY = 1;
  public double getMinX()
  {
    return minX;
  }
  public void setMinX(double x)
  {
    minX = x;
    changeAxes();
  }
  public double getMaxX()
  {
    return maxX;
  }
  public void setMaxX(double x)
  {
    maxX = x;
    changeAxes();
  }
  public double getMinY()
  {
    return minY;
  }
  public void setMinY(double y)
  {
    minY = y;
    changeAxes();
  }
  public double getMaxY()
  {
    return maxY;
  }
  public void setMaxY(double y)
  {
    maxY = y;
    changeAxes();
  }

  boolean centerTitle = true;
  public boolean getCenterTitle() { return centerTitle; }
  public void setCenterTitle(boolean b) { centerTitle = b; }

  Font titleFont;
  public Font getTitleFont() { return titleFont; }
  public void setTitleFont(Font f) { titleFont = f; }
  

  int axesNumberWidth = 30;
  public void setAxesNumberWidth(int w)
  {
    axesNumberWidth = w;
    changeAxes();
  }
  public int getAxesNumberWidth()
  {
    return axesNumberWidth;
  }

  int axesNumberHeight = 15;
  public void setAxesNumberHeight(int h)
  {
    axesNumberHeight = h;
    changeAxes();
  }
  public int getAxesNumberHeight()
  {
    return axesNumberHeight;
  }

  private Color graphBackgroundColor = Color.white;

  public Color getGraphBackground()
  {
    return graphBackgroundColor;
  }

  public void setGraphBackground(Color c)
  {
    graphBackgroundColor = c;
  }


  String title;
  public String getTitle() { return title; }
  public void setTitle(String s) { title = s; }


  boolean showAxes = true;
  public boolean getShowAxes() { return showAxes; }
  public void setShowAxes(boolean showAxes)
  {
    this.showAxes = showAxes;
    repaint();
  }

  boolean usePixelUnits = false;
  public boolean getUsePixelUnits() { return usePixelUnits; }
  public void setUsePixelUnits(boolean usePixelUnits)
  {
    this.usePixelUnits = usePixelUnits;
    repaint();
  }



  Vec clickedPosition = new Vec(2);
  public Vec getClickedPosition() { return clickedPosition; }
  public void setClickedPosition(Vec clickedPosition)
  {
    this.clickedPosition = clickedPosition;
    changeListeners.firePropertyChange(this, "clickedPosition", clickedPosition, clickedPosition);
  }

  Vec movedPosition = new Vec(2);
  public Vec getMovedPosition() { return movedPosition; }
  public void setMovedPosition(Vec moveedPosition)
  {
    this.movedPosition = movedPosition;
    changeListeners.firePropertyChange(this, "movedPosition", movedPosition, movedPosition);
  }

  public void mouseClicked(MouseEvent e) 
  {
    //System.out.println("figureaxes mouseClicked");
    clickedPosition.set(0, transformX(e.getX()));
    clickedPosition.set(1, transformY(e.getY()));
    setClickedPosition(clickedPosition);
  }

  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}

  int dragStartX, dragStartY;
  boolean drawingBox = false;
  public void mousePressed(MouseEvent e) 
  {
    showPopup(e);
    dragStartX = e.getX();
    dragStartY = e.getY();
    dragFinishX = dragStartX;
    dragFinishY = dragStartY;
  }

  public void mouseReleased(MouseEvent e) 
  {
    showPopup(e);
    if (drawingBox)
      {
	deleteRubberBox();
	drawingBox = false;

	// now compute new coordinates
	dragFinishX = e.getX();
	dragFinishY = e.getY();

	if (showAxes)
	  {
	    dragStartX -= axesNumberWidth;
	    dragFinishX -= axesNumberWidth;
	  }
	int tmp;

	if (dragStartX > dragFinishX)
	  {
	    tmp = dragFinishX;
	    dragFinishX = dragStartX;
	    dragStartX = tmp;
	  }
	if (dragStartY > dragFinishY)
	  {
	    tmp = dragFinishY;
	    dragFinishY = dragStartY;
	    dragStartY = tmp;
	  }

	if (((dragFinishX - dragStartX) < 10) ||
	    ((dragFinishY - dragStartY) < 10))
	  return;


	//System.out.println(minX + " " + maxX + " " + minY + " " + maxY + " " + graphRect);

	//System.out.println(dragStartX + " " + dragStartY + ",  " + dragFinishX + " " + dragFinishY);
	double newMinX = (maxX - minX)*(double) dragStartX/(double) graphRect.width + minX;
	double newMaxX = (maxX - minX)*(double) dragFinishX/(double) graphRect.width + minX;

	double newMinY = (maxY - minY)*(double) (graphRect.height - dragFinishY)/(double) graphRect.height + minY;
	double newMaxY = (maxY - minY)*(double) (graphRect.height - dragStartY)/(double)graphRect.height + minY;
	
	setMinX(newMinX);
	setMaxX(newMaxX);
	setMinY(newMinY);
	setMaxY(newMaxY);
	
	repaint();
      }
  }

  int dragFinishX, dragFinishY;
  
  public void mouseDragged(MouseEvent e)
  {
    if (e.isAltDown())
      {
	// shift coords
	double dx = transformX(e.getX()) - transformX(dragFinishX);
	double dy = transformY(e.getY()) - transformY(dragFinishY);
	
	setMinX(minX - dx);
	setMaxX(maxX - dx);
	setMinY(minY - dy);
	setMaxY(maxY - dy);
	repaint();
      }


    dragFinishX = e.getX();
    dragFinishY = e.getY();


    if (!e.isAltDown())
      {
	drawingBox = true;
	Rectangle r = fixBox(dragStartX, dragStartY, dragFinishX, dragFinishY);
	drawRubberBox(r);
      }
  }

  Rectangle fixBox(int startx, int starty, int x, int y)
  {
    int tmp;
    if (startx > x)
      {
	tmp = x;
	x = startx;
	startx = tmp;
      }
    if (starty > y)
      {
	tmp = y;
	y = starty;
	starty = tmp;
      }
    return new Rectangle(startx, starty, x - startx, y - starty);
  }

  public double transformX(int x)
  {
    if (showAxes) 
      x -= axesNumberWidth;
    return (maxX - minX)*x/graphRect.width+minX;
  }

  public double transformY(int y)
  {
    return (maxY - minY)*(graphRect.height - y)/graphRect.height+minY;
  }

  public void mouseMoved(MouseEvent e) 
  {
    movedPosition.set(0, transformX(e.getX()));
    movedPosition.set(1, transformY(e.getY()));
    setMovedPosition(movedPosition);
  }


  private void drawRubberBox(Rectangle box) 
  {
    Graphics g = getGraphics();
    g.setColor(Color.red);
    g.setXORMode(Color.blue);
    if (oldRubberBox != null) 
      {
	// doesn't work to just call deleteRubberBox here.
	g.drawRect(oldRubberBox.x, oldRubberBox.y, 
		   oldRubberBox.width, oldRubberBox.height);	
	oldRubberBox = null;
      }
    g.drawRect(box.x, box.y, box.width, box.height);	
    oldRubberBox = box;
  }

  private void deleteRubberBox() 
  {
    if (oldRubberBox != null) {
      Graphics g = getGraphics();
      if (g == null) {
	return;
      }
      //g.setColor(getBackground());
      g.setColor(Color.red);
      g.setXORMode(Color.blue);
      g.drawRect(oldRubberBox.x, oldRubberBox.y, 
		 oldRubberBox.width, oldRubberBox.height);	
      oldRubberBox = null;
    }
  }


  transient Rectangle oldRubberBox;

  protected void showPopup(MouseEvent e)
  {
    if (!e.isPopupTrigger()) return;
    popup.getPopupMenu().show(e.getComponent(), e.getX(), e.getY());
  }
  
  Popup popup = new Popup();
  class Popup extends JMenu implements ActionListener
  {
    public JMenuItem zoomOutMenuItem = new JMenuItem("zoom out");
    public JMenuItem zoomInMenuItem = new JMenuItem("zoom in");
    public Popup()
    {
      super();
      this.add(zoomOutMenuItem);
      zoomOutMenuItem.addActionListener(this);
      this.add(zoomInMenuItem);
      zoomInMenuItem.addActionListener(this);
    }

    public void actionPerformed(ActionEvent e)
    {
      JMenuItem mi = (JMenuItem)e.getSource();
      if (mi == zoomInMenuItem)
	{
	  double dx = (maxX - minX) / 4.0;
	  setMinX(minX + dx);
	  setMaxX(maxX - dx);
	  double dy = (maxY - minY) / 4.0;
	  setMinY(minY + dy);
	  setMaxY(maxY - dy);
	  FigureAxes.this.repaint();
	}
      else if (mi == zoomOutMenuItem)
	{
	  double dx = (maxX - minX) / 2.0;
	  setMinX(minX - dx);
	  setMaxX(maxX + dx);
	  double dy = (maxY - minY) / 2.0;
	  setMinY(minY - dy);
	  setMaxY(maxY + dy);
	  FigureAxes.this.repaint();
	}
    }

  }

  protected EventListenerList changeListeners = new EventListenerList();
  
  public void addPropertyChangeListener(PropertyChangeListener l)
  {
    changeListeners.add(l);
  }

  public void removePropertyChangeListener(PropertyChangeListener l)
  {
    changeListeners.remove(l);
  }



}


