package drew.figure;

import java.awt.*;
import drew.util.*;

public abstract class Plot extends Figure implements java.io.Serializable
{
  static final long serialVersionUID = 8632883868657051713L;

  // Implementing a concrete class amounts to specifying the domain/range
  // data class and wrappers
  public abstract int domainSize();
  public abstract double domainRef(int i);
  public abstract int rangeSize();
  public abstract double rangeRef(int i);
  /** When a domain is not specified, we assume 0..|range|. */
  public abstract void defaultDomain();

  
  Enum style = new Enum(new String[]{"LINE", "BAR", "SQUARE", "DIAMOND", "TRIANGLE", "SMALL_SQUARE", "POINT"});
  public void setStyle(Enum style)
  {
    this.style = style;
    if (enclosingAxes != null)
      enclosingAxes.repaint();
  }
  public Enum getStyle() { return style; }


  public void paint(Graphics g)
  {
    if (!live) return;

    if (rangeSize() == 0) return;  // no data!
    if (domainSize() == 0) defaultDomain(); 

    
    g.setColor(foreground);

    if (style.equals("LINE"))
      drawLineGraph(g);
    else if (style.equals("BAR"))
      drawBarGraph(g);
    else if (style.equals("SQUARE"))
      drawPointGraph(g,square);
    else if (style.equals("DIAMOND"))
      drawPointGraph(g,diamond);
    else if (style.equals("TRIANGLE"))
      drawPointGraph(g,triangle);
    else if (style.equals("SMALL_SQUARE"))
      drawPointGraph(g,smallSquare);
    else if (style.equals("POINT"))
      drawPointGraph(g,point);
  }

  private void drawLineGraph(Graphics g)
  {
    Point leftp = new Point(), rightp = new Point(), swap;
    transform(domainRef(0),rangeRef(0),leftp);
    for (int i = 0; i < domainSize() - 1; i++)
      {
	transform(domainRef(i+1),rangeRef(i+1),rightp);
	g.drawLine(leftp.x,leftp.y,rightp.x,rightp.y);
	swap = leftp;
	leftp = rightp;
	rightp = swap;
      }
  }

  private void drawBarGraph(Graphics g)
  {
    Point p = new Point();
    //int halfw = (int)(0.9*(double)graphRect.width/(double)(x.size()-1)/2.0);
    int halfw = (int)(0.9*(double)graphRect.width/(double)(maxX - minX)/2.0);
    for (int i = 0; i < domainSize(); i++)
      {
	transform(domainRef(i),rangeRef(i),p);
	g.fillRect(p.x - halfw,p.y,2*halfw,graphRect.height - p.y);
	//System.out.println(p.y + " " + (graphRect.height-p.y));
      }
  }

  private void drawPointGraph(Graphics g, Polygon poly)
  {
    Point p = new Point();
    g = g.create();
    for (int i = 0; i < domainSize(); i++)
      {	
        transform(domainRef(i),rangeRef(i),p);
	g.translate(p.x,p.y);
	g.drawPolygon(poly);
	g.translate(-p.x,-p.y);
      }
  }






  static Polygon square = makeSquare();
  static Polygon makeSquare()
  {
    int[] x = new int[4];
    int[] y = new int[4];
    x[0] = -2; y[0] = -2;
    x[1] = 2; y[1] = -2;
    x[2] = 2; y[2] = 2;
    x[3] = -2; y[3] = 2;
    return(new Polygon(x,y,4));
  }

  static Polygon diamond = makeDiamond();
  static Polygon makeDiamond()
  {
    int[] x = new int[4];
    int[] y = new int[4];
    x[0] = 0; y[0] = 2;
    x[1] = 2; y[1] = 0;
    x[2] = 0; y[2] = -2;
    x[3] = -2; y[3] = 0;
    return(new Polygon(x,y,4));
  }
    
  static Polygon triangle = makeTriangle();
  static Polygon makeTriangle()
  {
    int[] x = new int[3];
    int[] y = new int[3];
    x[0] = -2; y[0] = 2;
    x[1] = 2; y[1] = 2;
    x[2] = 0; y[2] = -2;
    return(new Polygon(x,y,3));
  }

  static Polygon smallSquare = makeSmallSquare();
  static Polygon makeSmallSquare()
  {
    int[] x = new int[4];
    int[] y = new int[4];
    x[0] = -1; y[0] = -1;
    x[1] = 1; y[1] = -1;
    x[2] = 1; y[2] = 1;
    x[3] = -1; y[3] = 1;
    return(new Polygon(x,y,4));
  }

  static Polygon point = makePoint();
  static Polygon makePoint()
  {
    int[] x = new int[1];
    int[] y = new int[1];
    x[0] = 0; y[0] = 0;
    return(new Polygon(x,y,1));
  }


}
