package math;

import java.text.*;

/** Provides a method to format doubles such that the resulting
    strings may be arranged in neat little columns and rows.
*/
public class Format
{
  static DecimalFormat df, dfScientific;

  static 
  {
    df = new DecimalFormat("#.####");
    df.setGroupingUsed(false);

    dfScientific = new DecimalFormat("#.####E0");
    dfScientific.setGroupingUsed(false);
  }

  public static String format(double d)
  {
    return format(d, false);
  }

  public static String format(double d, boolean pad)
  {
    String s;
    if (d == 0)
      s = "0";
    else if (Double.isNaN(d))
      s = "NaN";
    else if (Double.isInfinite(d))
      {
	if (d > 0)
	  s = "+Inf";
	else
	  s = "-Inf";
      }
    else 
      {
	double m = Math.abs(d);
	if ((m >= 1.0E-4) && (m < 10E5))
	  {
	    // worst case:  -10000.1234 is length 12
	    s = df.format(d);
	  }
	else
	  {
	    // worst case: -x.xxxxE-xxx is length 12
	    s = dfScientific.format(d);
	  }
      }

    // pad out, flush right
    if (pad)
      while (s.length() < 12)  
	s = " " + s;
    
    return s;
  }


  public static void main(String args[])
  {
    System.out.println(Format.format(100));
    System.out.println(Format.format(10000000000.0));
    System.out.println(Format.format(0.1));
    System.out.println(Format.format(0.0000000000012345));
    System.out.println(Format.format(1.0E9));
    System.out.println(Format.format(1.0E-9));
    System.out.println(Format.format(100000000000.0));
    System.out.println(Format.format(-100.24568989289283928392839238));
    System.out.println(Format.format(0.00000000000024568989289283928392839238));
    System.out.println(Format.format( 1000000.0));
    System.out.println(Format.format(-100000.123456));
    System.out.println(Format.format(-1000000.123456));
    System.out.println(Format.format(0.001));
    System.out.println(Format.format(0.0001));
    System.out.println(Format.format(0.00001));
    System.out.println(Format.format(0.000001));
    System.out.println(Format.format(Double.POSITIVE_INFINITY));
    System.out.println(Format.format(Double.NEGATIVE_INFINITY));
    System.out.println(Format.format(Double.NaN));
  }

}
