package math.pdf;

import java.util.Vector;
import java.beans.*;
import drew.util.*;
import math.*;

public abstract class Pdf 
  implements java.io.Serializable
				     
{
  static final long serialVersionUID = -5571418152974526414L;

  protected EventListenerList changeListeners = new EventListenerList();
  
  public void addPropertyChangeListener(PropertyChangeListener l)
  {
    changeListeners.add(l);
  }

  public void removePropertyChangeListener(PropertyChangeListener l)
  {
    changeListeners.remove(l);
  }

  private double value;
  public double getValue() { return value; }
  protected void setValue(double value)
  {
    this.value = value;
    if (changeListeners.hasListeners())
      changeListeners.firePropertyChange(this, "value", new Double(value), new Double(value));
  }
 
  protected Vec input = new Vec(3);
  public Vec getInput() { return input; }
  public void setInput(Vec input)
  {
    this.input = input;
    if (changeListeners.hasListeners())
      changeListeners.firePropertyChange(this, "input", input, input);
    update();
  }

  private boolean autoUpdate = true;
  public boolean getAutoUpdate() { return autoUpdate; }
  public void setAutoUpdate(boolean autoUpdate)
  {
    this.autoUpdate = autoUpdate;
  }

  protected double evaluate()
  {
    return evaluate(input);
  }

  public abstract double evaluate(Vec input);


  public double evaluateLog()
  {
    return evaluateLog(input);
  }

  // override for more efficient implementation
  public double evaluateLog(Vec input)
  {
    return Math.log(evaluate(input));
  }

  public Vec evaluate(IndexedVecs input)
  {
    Vec out = new Vec(input.size());
    for (int i = 0; i < input.size(); i++)
      {
	out.set(i, evaluate(input.ref(i)));
      }
    return out;
  }

  protected IndexedVecs trainingData;
  public IndexedVecs getTrainingData() { return trainingData; }
  public void setTrainingData(IndexedVecs trainingData) 
  {
    this.trainingData = trainingData;
  }
  protected IndexedDoubles w;
  public IndexedDoubles getw() { return w; }
  public void setw(IndexedDoubles w) 
  {
    this.w = w;
  }

  public abstract void train();

  public abstract Vec sample();

  protected void update()
  {
    if (autoUpdate)
      setValue(evaluate());
  }



}

